#include "testlib.h"
#include <cstdio>
#include <cmath>
#include <cstdlib>
#include <cassert>
#include <ctime>
#include <cstring>
#include <string>
#include <set>
#include <map>
#include <vector>
#include <list>
#include <deque>
#include <queue>
#include <sstream>
#include <iostream>
#include <algorithm>

typedef long long ll;

using namespace std;

const int GROUPS = 4;
const int MAXN[] = { 100 * 1000, 10, 100, 100, 100 * 1000 };
const int MAXA[] = { 10, 10, 100, 1000 * 1000 * 1000, 1000 * 1000 * 1000 };

const int INF = 1000 * 1000 * 1000 + 21;
const ll LLINF = (1ll << 60) + 5;
const int MOD = 1000 * 1000 * 1000 + 7;

const int MAX_N = 100 * 1000 + 5;
const int MAX_ABS = 1000 * 1000 * 1000;

int n;
int arr[MAX_N];
char st[MAX_N];

ll solve()
{
    int up = INF;
    int dw = -INF;

    for (int i = 4; i < n; ++i)
    {
        bool all_same = true;
        for (int j = i - 4; j < i; ++j) all_same &= st[j];

        if (all_same)
        {
            if (st[i]) dw = max(dw, *min_element(arr + i - 4, arr + i + 1));
            else up = min(up, *min_element(arr + i - 4, arr + i + 1) - 1);
        }
    }

    ensuref(dw <= up, "Solution must exist");
    
    if (up < -MAX_ABS || MAX_ABS < dw) ensuref(false, "Solution must exist");
    
    return min(up, MAX_ABS);
}

string getString(int tmin, int tmax) {
    if (n < 5) {
        string res = "";
        for (int i = 0; i < n; i++) {
            res += "0";
        }
        
        return res;
    }

    string res = "0000";
    bool turnedIn = false;
    for (int i = 4; i < n; i++) {
        int mn = MAX_ABS;
        int mx = -MAX_ABS;
        bool allZeroes = true;
        bool allOnes = true;
        for (int j = i - 4; j <= i; j++) {
            mn = min(mn, arr[j]);
            mx = max(mx, arr[j]);

            if (j < i && res[j] == '1') {
                allZeroes = false;
            }
            if (j < i && res[j] == '0') {
                allOnes = false;
            }
        }

        if (mx < tmin && allZeroes) {
            turnedIn = true;
            res += "1";
        } else if (mn > tmax && allOnes) {
            turnedIn = false;
            res += "0";
        } else if (turnedIn) {
            res += "1";
        } else {
            res += "0";
        }
    }
    return res;
}

std::string ParseGroup(InStream& test, Validator& validator) {
    static const char rareChar = '%';
    static const std::string randomString = "forever_papich";
    static const std::string groupToken = "GROUP: ";
    static const char terminalToken = '#';

    if (!validator.group().empty()) {
        return validator.group(); // Okay, we have some group
    }

    std::string toParse = randomString + groupToken;

    std::string read;
    
    {
        char c = test.readChar();
        if (c != rareChar) {
            test.unreadChar(c);
            return std::string();
        }
    }
    
    for (size_t i = 0; i < toParse.size(); i++) {
        if (!test.eof()) {
            read += test.readChar();
        }
    }

    ensuref(!test.eof() && read == toParse, "Bad fake group format");

    std::string group;

    char c = test.readChar();
    while (c != terminalToken) {
        group += c;
        ensuref(!test.eof(), "Bad fake group format");
        c = test.readChar();
    }

    test.readEoln();
    return group;
}

int main(int argc, char ** argv) {
    registerValidation(argc, argv);
    
    string groupS = ParseGroup(inf, validator);
    
    ensuref(groupS.size() == 1 && 
            '0' <= groupS.data()[0] &&
            groupS.data()[0] <= '4', "Wrong group");
            
    int group = atoi(groupS.data());
    ensuref(0 <= group && group <= GROUPS, "Wrong group number");

    n = inf.readInt(5, MAXN[group], "n");
    inf.readEoln();

    for (int i = 0; i < n; i++) {
        arr[i] = inf.readInt(-MAXA[group], MAXA[group], "t_" + vtos(i + 1));
        if (i == n - 1)
            inf.readEoln();
        else
            inf.readSpace();
    }

    string s = inf.readToken("[01]{" + vtos(n) + "}");
    inf.readEoln();

    for (int i = 0; i < n; i++) {
        st[i] = s[i] - '0';
    }

    ll mx = solve();

    for (int i = 0; i < n; ++i) st[i] ^= 1;
    for (int i = 0; i < n; ++i) arr[i] = -arr[i];

    ll mn = -solve();
    
    for (int i = 0; i < n; ++i) st[i] ^= 1;
    for (int i = 0; i < n; ++i) arr[i] = -arr[i];
    
    ensuref(mn <= mx, "Solution must exist");

    ensuref(abs(mn) <= MAX_ABS && abs(mx) <= MAX_ABS, "The absolute value of the answer should be no more than %d", MAX_ABS);

    string res = getString(mn, mx);
    ensuref(s == res, "the template is %s, but %s found", s.c_str(), res.c_str());

    inf.readEof();

    return 0;
}
